package ru.dbotthepony.mc.otm;

import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.InterModComms;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.event.lifecycle.InterModEnqueueEvent;
import net.minecraftforge.fml.event.lifecycle.InterModProcessEvent;
import net.minecraftforge.fmlserverevents.FMLServerStartedEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import ru.dbotthepony.mc.otm.capability.AndroidCapability;
import ru.dbotthepony.mc.otm.capability.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.client.AndroidGui;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.matter.MatterRegistry;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.shapes.Point;
import ru.dbotthepony.mc.otm.shapes.Rectangle;

import java.util.stream.Collectors;

// The value here should match an entry in the META-INF/mods.toml file
@Mod(OverdriveThatMatters.MOD_ID)
public class OverdriveThatMatters {
	// Directly reference a log4j logger.
	public static final String MOD_ID = "overdrive_that_matters";
	public static final Logger LOGGER = LogManager.getLogger();

	public static AndroidGui ANDROID_GUI;

	public static CreativeModeTab CREATIVE_TAB;

	public OverdriveThatMatters() {
		// Register the setup method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setup);
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setupClient);
		// Register the enqueueIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::enqueueIMC);
		// Register the processIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::processIMC);

		// Register ourselves for server and other game events we are interested in
		MinecraftForge.EVENT_BUS.register(this);
		MinecraftForge.EVENT_BUS.register(AndroidCapabilityPlayer.class);
		MinecraftForge.EVENT_BUS.register(AndroidCapability.class);
		MinecraftForge.EVENT_BUS.register(MatterGrid.class);

		// LOGGER.info("Registered event handlers");

		CREATIVE_TAB = new CreativeModeTab("otm") {
			@Override
			public ItemStack makeIcon() {
				return new ItemStack(Registry.Items.BATTERY_CREATIVE, 1);
			}
		};
	}

	private void setup(final FMLCommonSetupEvent event) {
		// some preinit code
		MatteryCapability.register();
		// LOGGER.info("Registered capabilities");
		MatteryNetworking.register();
		// LOGGER.info("Registered network");

		MatterRegistry.registerInitialItems();
	}

	private void setupClient(final FMLClientSetupEvent event) {
		ANDROID_GUI = new AndroidGui();
		MinecraftForge.EVENT_BUS.register(ANDROID_GUI);
		MinecraftForge.EVENT_BUS.register(MatterRegistry.class);

		Registry.Menus.registerScreens(event);
	}

	private void enqueueIMC(final InterModEnqueueEvent event) {
		// some example code to dispatch IMC to another mod
		InterModComms.sendTo("examplemod", "helloworld", () -> { LOGGER.info("Hello world from the MDK"); return "Hello world";});
	}

	private void processIMC(final InterModProcessEvent event) {
		// some example code to receive and process InterModComms from other mods
		LOGGER.info("Got IMC {}", event.getIMCStream().
				map(m->m.messageSupplier().get()).
				collect(Collectors.toList()));
	}

	// You can use SubscribeEvent and let the Event Bus discover methods to call
	@SubscribeEvent
	public void onServerStarting(FMLServerStartedEvent event) {
		// do something when the server starts
	}

	@Mod.EventBusSubscriber(bus = Mod.EventBusSubscriber.Bus.MOD)
	public static class RegistryEventSubscriber {
		@SubscribeEvent
		public static void blocks(final RegistryEvent.Register<Block> event) {
			Registry.Blocks.register(event);
		}

		@SubscribeEvent
		public static void items(final RegistryEvent.Register<Item> event) {
			Registry.Items.register(event);
		}

		@SubscribeEvent
		public static void tiles(final RegistryEvent.Register<BlockEntityType<?>> event) {
			Registry.BlockEntities.register(event);
		}

		@SubscribeEvent
		public static void menus(final RegistryEvent.Register<MenuType<?>> event) {
			Registry.Menus.register(event);
		}
	}
}
