package ru.dbotthepony.mc.otm;

import net.minecraft.client.gui.screens.MenuScreens;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import ru.dbotthepony.mc.otm.block.*;
import ru.dbotthepony.mc.otm.block.entity.*;
import ru.dbotthepony.mc.otm.item.ItemBattery;
import ru.dbotthepony.mc.otm.item.ItemMatterCapacitor;
import ru.dbotthepony.mc.otm.item.ItemPatternStorage;
import ru.dbotthepony.mc.otm.item.ItemPill;
import ru.dbotthepony.mc.otm.menu.*;
import ru.dbotthepony.mc.otm.screen.*;

import java.math.BigDecimal;

public class Registry {
	public static final DamageSource DAMAGE_BECOME_ANDROID = new DamageSource("otmBecomeAndroid");
	public static final DamageSource DAMAGE_BECOME_HUMANE = new DamageSource("otmBecomeHumane");

	static {
		DAMAGE_BECOME_ANDROID.bypassArmor().bypassInvul().bypassMagic();
		DAMAGE_BECOME_HUMANE.bypassArmor().bypassInvul().bypassMagic();
	}

	public static class Names {
		public static final ResourceLocation ANDROID_STATION = new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_station");
		public static final ResourceLocation BATTERY_BANK = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_bank");
		public static final ResourceLocation MATTER_DECOMPOSER = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_decomposer");
		public static final ResourceLocation MATTER_CAPACITOR_BANK = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_capacitor_bank");
		public static final ResourceLocation MATTER_CABLE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_cable");
		public static final ResourceLocation PATTERN_STORAGE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "pattern_storage");
		public static final ResourceLocation MATTER_SCANNER = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_scanner");
		public static final ResourceLocation MATTER_PANEL = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_panel");
		public static final ResourceLocation MATTER_REPLICATOR = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_replicator");

		public static final ResourceLocation ANDROID_CAPABILITY = new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_capability");

		public static final ResourceLocation PILL_ANDROID = new ResourceLocation(OverdriveThatMatters.MOD_ID, "pill_android");
		public static final ResourceLocation PILL_HUMANE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "pill_humane");

		public static final ResourceLocation BATTERY_CRUDE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_crude");
		public static final ResourceLocation BATTERY_BASIC = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_basic");
		public static final ResourceLocation BATTERY_NORMAL = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_normal");
		public static final ResourceLocation BATTERY_DENSE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_dense");
		public static final ResourceLocation BATTERY_CAPACITOR = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_capacitor");
		public static final ResourceLocation BATTERY_CREATIVE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "battery_creative");

		public static final ResourceLocation MATTER_CAPACITOR_NORMAL = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_capacitor_normal");
		public static final ResourceLocation MATTER_CAPACITOR_CREATIVE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_capacitor_creative");

		public static final ResourceLocation PATTERN_DRIVE_NORMAL = new ResourceLocation(OverdriveThatMatters.MOD_ID, "pattern_drive_normal");
		public static final ResourceLocation PATTERN_DRIVE_CREATIVE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "pattern_drive_creative");
	}

	public static class Blocks {
		public static final Block ANDROID_STATION = new BlockAndroidStation();
		public static final Block BATTERY_BANK = new BlockBatteryBank();
		public static final Block MATTER_DECOMPOSER = new BlockMatterDecomposer();
		public static final Block MATTER_CAPACITOR_BANK = new BlockMatterCapacitorBank();
		public static final Block MATTER_CABLE = new BlockMatterCable();
		public static final Block PATTERN_STORAGE = new BlockPatternStorage();
		public static final Block MATTER_SCANNER = new BlockMatterScanner();
		public static final Block MATTER_PANEL = new BlockMatterPanel();
		public static final Block MATTER_REPLICATOR = new BlockMatterReplicator();

		static {
			ANDROID_STATION.setRegistryName(Names.ANDROID_STATION);
			BATTERY_BANK.setRegistryName(Names.BATTERY_BANK);
			MATTER_DECOMPOSER.setRegistryName(Names.MATTER_DECOMPOSER);
			MATTER_CAPACITOR_BANK.setRegistryName(Names.MATTER_CAPACITOR_BANK);
			MATTER_CABLE.setRegistryName(Names.MATTER_CABLE);
			PATTERN_STORAGE.setRegistryName(Names.PATTERN_STORAGE);
			MATTER_SCANNER.setRegistryName(Names.MATTER_SCANNER);
			MATTER_PANEL.setRegistryName(Names.MATTER_PANEL);
			MATTER_REPLICATOR.setRegistryName(Names.MATTER_REPLICATOR);
		}

		public static void register(final RegistryEvent.Register<Block> event) {
			event.getRegistry().register(ANDROID_STATION);
			event.getRegistry().register(BATTERY_BANK);
			event.getRegistry().register(MATTER_DECOMPOSER);
			event.getRegistry().register(MATTER_CAPACITOR_BANK);
			event.getRegistry().register(MATTER_CABLE);
			event.getRegistry().register(PATTERN_STORAGE);
			event.getRegistry().register(MATTER_SCANNER);
			event.getRegistry().register(MATTER_PANEL);
			event.getRegistry().register(MATTER_REPLICATOR);

			// OverdriveThatMatters.LOGGER.info("Registered blocks");
		}
	}

	public static class Items {
		public static final Item ANDROID_STATION = new BlockItem(Blocks.ANDROID_STATION, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item BATTERY_BANK = new BlockItem(Blocks.BATTERY_BANK, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_DECOMPOSER = new BlockItem(Blocks.MATTER_DECOMPOSER, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_CAPACITOR_BANK = new BlockItem(Blocks.MATTER_CAPACITOR_BANK, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_CABLE = new BlockItem(Blocks.MATTER_CABLE, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item PATTERN_STORAGE = new BlockItem(Blocks.PATTERN_STORAGE, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_SCANNER = new BlockItem(Blocks.MATTER_SCANNER, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_PANEL = new BlockItem(Blocks.MATTER_PANEL, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));
		public static final Item MATTER_REPLICATOR = new BlockItem(Blocks.MATTER_REPLICATOR, new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.CREATIVE_TAB));

		public static final ItemPill PILL_ANDROID = new ItemPill(ItemPill.PillType.BECOME_ANDROID);
		public static final ItemPill PILL_HUMANE = new ItemPill(ItemPill.PillType.BECOME_HUMANE);

		public static final ItemBattery BATTERY_CRUDE = new ItemBattery(new BigDecimal(30_000), new BigDecimal(150), new BigDecimal(150));
		public static final ItemBattery BATTERY_BASIC = new ItemBattery(new BigDecimal(60_000), new BigDecimal(300), new BigDecimal(300));
		public static final ItemBattery BATTERY_NORMAL = new ItemBattery(new BigDecimal(250_000), new BigDecimal(1000), new BigDecimal(1000));
		public static final ItemBattery BATTERY_DENSE = new ItemBattery(new BigDecimal(1_000_000), new BigDecimal(2000), new BigDecimal(2000));
		public static final ItemBattery BATTERY_CAPACITOR = new ItemBattery(new BigDecimal(150_000), new BigDecimal(15000), new BigDecimal(15000));
		public static final ItemBattery BATTERY_CREATIVE = new ItemBattery();

		public static final ItemMatterCapacitor MATTER_CAPACITOR_NORMAL = new ItemMatterCapacitor(new BigDecimal("10"));
		public static final ItemMatterCapacitor MATTER_CAPACITOR_CREATIVE = new ItemMatterCapacitor();

		public static final ItemPatternStorage PATTERN_DRIVE_NORMAL = new ItemPatternStorage(4);
		public static final ItemPatternStorage PATTERN_DRIVE_CREATIVE = new ItemPatternStorage();

		static {
			ANDROID_STATION.setRegistryName(Names.ANDROID_STATION);
			BATTERY_BANK.setRegistryName(Names.BATTERY_BANK);
			MATTER_DECOMPOSER.setRegistryName(Names.MATTER_DECOMPOSER);
			MATTER_CAPACITOR_BANK.setRegistryName(Names.MATTER_CAPACITOR_BANK);
			MATTER_CABLE.setRegistryName(Names.MATTER_CABLE);
			PATTERN_STORAGE.setRegistryName(Names.PATTERN_STORAGE);
			MATTER_SCANNER.setRegistryName(Names.MATTER_SCANNER);
			MATTER_PANEL.setRegistryName(Names.MATTER_PANEL);
			MATTER_REPLICATOR.setRegistryName(Names.MATTER_REPLICATOR);

			PILL_ANDROID.setRegistryName(Names.PILL_ANDROID);
			PILL_HUMANE.setRegistryName(Names.PILL_HUMANE);
			BATTERY_CRUDE.setRegistryName(Names.BATTERY_CRUDE);
			BATTERY_BASIC.setRegistryName(Names.BATTERY_BASIC);
			BATTERY_NORMAL.setRegistryName(Names.BATTERY_NORMAL);
			BATTERY_DENSE.setRegistryName(Names.BATTERY_DENSE);
			BATTERY_CAPACITOR.setRegistryName(Names.BATTERY_CAPACITOR);
			BATTERY_CREATIVE.setRegistryName(Names.BATTERY_CREATIVE);

			MATTER_CAPACITOR_NORMAL.setRegistryName(Names.MATTER_CAPACITOR_NORMAL);
			MATTER_CAPACITOR_CREATIVE.setRegistryName(Names.MATTER_CAPACITOR_CREATIVE);

			PATTERN_DRIVE_NORMAL.setRegistryName(Names.PATTERN_DRIVE_NORMAL);
			PATTERN_DRIVE_CREATIVE.setRegistryName(Names.PATTERN_DRIVE_CREATIVE);
		}

		public static void register(final RegistryEvent.Register<Item> event) {
			event.getRegistry().register(ANDROID_STATION);
			event.getRegistry().register(BATTERY_BANK);
			event.getRegistry().register(MATTER_DECOMPOSER);
			event.getRegistry().register(MATTER_CAPACITOR_BANK);
			event.getRegistry().register(MATTER_CABLE);
			event.getRegistry().register(PATTERN_STORAGE);
			event.getRegistry().register(MATTER_SCANNER);
			event.getRegistry().register(MATTER_PANEL);
			event.getRegistry().register(MATTER_REPLICATOR);

			event.getRegistry().register(PILL_ANDROID);
			event.getRegistry().register(PILL_HUMANE);
			event.getRegistry().register(BATTERY_CRUDE);
			event.getRegistry().register(BATTERY_BASIC);
			event.getRegistry().register(BATTERY_NORMAL);
			event.getRegistry().register(BATTERY_DENSE);
			event.getRegistry().register(BATTERY_CAPACITOR);
			event.getRegistry().register(BATTERY_CREATIVE);

			event.getRegistry().register(MATTER_CAPACITOR_NORMAL);
			event.getRegistry().register(MATTER_CAPACITOR_CREATIVE);

			event.getRegistry().register(PATTERN_DRIVE_NORMAL);
			event.getRegistry().register(PATTERN_DRIVE_CREATIVE);

			// OverdriveThatMatters.LOGGER.info("Registered items");
		}
	}

	public static class BlockEntities {
		public static final BlockEntityType<BlockEntityAndroidStation> ANDROID_STATION = BlockEntityType.Builder.of(BlockEntityAndroidStation::new, Blocks.ANDROID_STATION).build(null);
		public static final BlockEntityType<BlockEntityBatteryBank> BATTERY_BANK = BlockEntityType.Builder.of(BlockEntityBatteryBank::new, Blocks.BATTERY_BANK).build(null);
		public static final BlockEntityType<BlockEntityMatterDecomposer> MATTER_DECOMPOSER = BlockEntityType.Builder.of(BlockEntityMatterDecomposer::new, Blocks.MATTER_DECOMPOSER).build(null);
		public static final BlockEntityType<BlockEntityMatterCapacitorBank> MATTER_CAPACITOR_BANK = BlockEntityType.Builder.of(BlockEntityMatterCapacitorBank::new, Blocks.MATTER_CAPACITOR_BANK).build(null);
		public static final BlockEntityType<BlockEntityMatterCable> MATTER_CABLE = BlockEntityType.Builder.of(BlockEntityMatterCable::new, Blocks.MATTER_CABLE).build(null);
		public static final BlockEntityType<BlockEntityPatternStorage> PATTERN_STORAGE = BlockEntityType.Builder.of(BlockEntityPatternStorage::new, Blocks.PATTERN_STORAGE).build(null);
		public static final BlockEntityType<BlockEntityMatterScanner> MATTER_SCANNER = BlockEntityType.Builder.of(BlockEntityMatterScanner::new, Blocks.MATTER_SCANNER).build(null);
		public static final BlockEntityType<BlockEntityMatterPanel> MATTER_PANEL = BlockEntityType.Builder.of(BlockEntityMatterPanel::new, Blocks.MATTER_PANEL).build(null);
		public static final BlockEntityType<BlockEntityMatterReplicator> MATTER_REPLICATOR = BlockEntityType.Builder.of(BlockEntityMatterReplicator::new, Blocks.MATTER_REPLICATOR).build(null);

		static {
			ANDROID_STATION.setRegistryName(Names.ANDROID_STATION);
			BATTERY_BANK.setRegistryName(Names.BATTERY_BANK);
			MATTER_DECOMPOSER.setRegistryName(Names.MATTER_DECOMPOSER);
			MATTER_CAPACITOR_BANK.setRegistryName(Names.MATTER_CAPACITOR_BANK);
			MATTER_CABLE.setRegistryName(Names.MATTER_CABLE);
			PATTERN_STORAGE.setRegistryName(Names.PATTERN_STORAGE);
			MATTER_SCANNER.setRegistryName(Names.MATTER_SCANNER);
			MATTER_PANEL.setRegistryName(Names.MATTER_PANEL);
			MATTER_REPLICATOR.setRegistryName(Names.MATTER_REPLICATOR);
		}

		public static void register(final RegistryEvent.Register<BlockEntityType<?>> event) {
			event.getRegistry().register(ANDROID_STATION);
			event.getRegistry().register(BATTERY_BANK);
			event.getRegistry().register(MATTER_DECOMPOSER);
			event.getRegistry().register(MATTER_CAPACITOR_BANK);
			event.getRegistry().register(MATTER_CABLE);
			event.getRegistry().register(PATTERN_STORAGE);
			event.getRegistry().register(MATTER_SCANNER);
			event.getRegistry().register(MATTER_PANEL);
			event.getRegistry().register(MATTER_REPLICATOR);

			// OverdriveThatMatters.LOGGER.info("Registered block entities");
		}
	}

	public static class Menus {
		public static final MenuType<AndroidStationMenu> ANDROID_STATION = new MenuType<>(AndroidStationMenu::new);
		public static final MenuType<BatteryBankMenu> BATTERY_BANK = new MenuType<>(BatteryBankMenu::new);
		public static final MenuType<MatterDecomposerMenu> MATTER_DECOMPOSER = new MenuType<>(MatterDecomposerMenu::new);
		public static final MenuType<MatterCapacitorBankMenu> MATTER_CAPACITOR_BANK = new MenuType<>(MatterCapacitorBankMenu::new);
		public static final MenuType<PatternStorageMenu> PATTERN_STORAGE = new MenuType<>(PatternStorageMenu::new);
		public static final MenuType<MatterScannerMenu> MATTER_SCANNER = new MenuType<>(MatterScannerMenu::new);
		public static final MenuType<MatterPanelMenu> MATTER_PANEL = new MenuType<>(MatterPanelMenu::new);
		public static final MenuType<MatterReplicatorMenu> MATTER_REPLICATOR = new MenuType<>(MatterReplicatorMenu::new);

		static {
			ANDROID_STATION.setRegistryName(Names.ANDROID_STATION);
			BATTERY_BANK.setRegistryName(Names.BATTERY_BANK);
			MATTER_DECOMPOSER.setRegistryName(Names.MATTER_DECOMPOSER);
			MATTER_CAPACITOR_BANK.setRegistryName(Names.MATTER_CAPACITOR_BANK);
			PATTERN_STORAGE.setRegistryName(Names.PATTERN_STORAGE);
			MATTER_SCANNER.setRegistryName(Names.MATTER_SCANNER);
			MATTER_PANEL.setRegistryName(Names.MATTER_PANEL);
			MATTER_REPLICATOR.setRegistryName(Names.MATTER_REPLICATOR);
		}

		public static void register(final RegistryEvent.Register<MenuType<?>> event) {
			event.getRegistry().register(ANDROID_STATION);
			event.getRegistry().register(BATTERY_BANK);
			event.getRegistry().register(MATTER_DECOMPOSER);
			event.getRegistry().register(MATTER_CAPACITOR_BANK);
			event.getRegistry().register(PATTERN_STORAGE);
			event.getRegistry().register(MATTER_SCANNER);
			event.getRegistry().register(MATTER_PANEL);
			event.getRegistry().register(MATTER_REPLICATOR);

			// OverdriveThatMatters.LOGGER.info("Registered menus");
		}

		public static void registerScreens(final FMLClientSetupEvent event) {
			MenuScreens.register(ANDROID_STATION, AndroidStationScreen::new);
			MenuScreens.register(BATTERY_BANK, BatteryBankScreen::new);
			MenuScreens.register(MATTER_DECOMPOSER, MatterDecomposerScreen::new);
			MenuScreens.register(MATTER_CAPACITOR_BANK, MatterCapacitorBankScreen::new);
			MenuScreens.register(PATTERN_STORAGE, PatternStorageScreen::new);
			MenuScreens.register(MATTER_SCANNER, MatterScannerScreen::new);
			MenuScreens.register(MATTER_PANEL, MatterPanelScreen::new);
			MenuScreens.register(MATTER_REPLICATOR, MatterReplicatorScreen::new);

			// OverdriveThatMatters.LOGGER.info("Registered screens");
		}
	}
}
